Package: celldex
Title: Index of Reference Cell Type Datasets
Version: 1.20.0
Date: 2024-04-25
Authors@R: c(person("Dvir", "Aran", email="dvir.aran@ucsf.edu", role=c("aut")),
    person("Aaron", "Lun", email="infinite.monkeys.with.keyboards@gmail.com", role=c("aut", "cre", "cph")),
    person("Daniel", "Bunis", role="aut"),
    person("Jared", "Andrews", email = "jared.andrews07@gmail.com", role="aut"),
    person("Friederike", "Dündar", email = "frd2007@med.cornell.edu", role="aut"))
Description: 
    Provides a collection of reference expression datasets with curated cell
    type labels, for use in procedures like automated annotation of single-cell
    data or deconvolution of bulk RNA-seq.
License: GPL-3
Depends: SummarizedExperiment
Imports: utils, methods, Matrix, ExperimentHub, AnnotationHub,
        AnnotationDbi, S4Vectors, DelayedArray, DelayedMatrixStats,
        gypsum, alabaster.base, alabaster.matrix, alabaster.se, DBI,
        RSQLite, jsonlite
Suggests: testthat, knitr, rmarkdown, BiocStyle, DT, jsonvalidate,
        BiocManager, ensembldb
biocViews: ExperimentHub, ExperimentData, ExpressionData,
        SequencingData, RNASeqData
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/LTLA/celldex
BugReports: https://support.bioconductor.org/
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/celldex
git_branch: RELEASE_3_22
git_last_commit: 4db42c8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 16:11:13 UTC; biocbuild
Author: Dvir Aran [aut],
  Aaron Lun [aut, cre, cph],
  Daniel Bunis [aut],
  Jared Andrews [aut],
  Friederike Dündar [aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>

# Generated by roxygen2: do not edit by hand

export(BlueprintEncodeData)
export(DatabaseImmuneCellExpressionData)
export(HumanPrimaryCellAtlasData)
export(ImmGenData)
export(MonacoImmuneData)
export(MouseRNAseqData)
export(NovershternHematopoieticData)
export(defineTextQuery)
export(fetchLatestVersion)
export(fetchMetadata)
export(fetchReference)
export(listReferences)
export(listVersions)
export(saveReference)
export(searchReferences)
export(surveyReferences)
import(methods)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(Matrix,lgCMatrix)
importClassesFrom(alabaster.matrix,ReloadedArray)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationHub,AnnotationHub)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbGetQuery)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,is_sparse)
importFrom(DelayedArray,type)
importFrom(DelayedMatrixStats,colAnyNAs)
importFrom(DelayedMatrixStats,rowAnyNAs)
importFrom(ExperimentHub,ExperimentHub)
importFrom(RSQLite,SQLite)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(alabaster.base,altReadObject)
importFrom(alabaster.base,altReadObjectFunction)
importFrom(alabaster.base,readObject)
importFrom(alabaster.base,readObjectFile)
importFrom(alabaster.base,saveObject)
importFrom(gypsum,cacheDirectory)
importFrom(gypsum,defineTextQuery)
importFrom(gypsum,fetchLatest)
importFrom(gypsum,fetchMetadataDatabase)
importFrom(gypsum,fetchMetadataSchema)
importFrom(gypsum,formatObjectMetadata)
importFrom(gypsum,listAssets)
importFrom(gypsum,saveFile)
importFrom(gypsum,saveVersion)
importFrom(gypsum,searchMetadataTextFilter)
importFrom(gypsum,validateMetadata)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(utils,read.delim)
importMethodsFrom(alabaster.se,saveObject)

#' Obtain bulk microarray expression for sorted hematopoietic cells 
#'
#' Download and cache the normalized expression values of 211 bulk human microarray samples
#' of sorted hematopoietic cell populations that can be found in 
#' \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE24759}{GSE24759}.
#'
#' @inheritParams HumanPrimaryCellAtlasData
#'
#' @details 
#' The dataset contains 211 human microarray samples annotated to 16 main cell types (\code{"label.main"}):
#' \itemize{
#'     \item Basophils
#'     \item B cells
#'     \item CMPs
#'     \item Dendritic cells
#'     \item Eosinophils
#'     \item Erythroid cells
#'     \item GMPS
#'     \item Granulocytes
#'     \item HSCs
#'     \item Megakaryocytes
#'     \item MEPs
#'     \item Monocytes
#'     \item NK cells
#'     \item NK T cells
#'     \item CD8+ T cells
#'     \item CD4+ T cells
#' }
#'
#' Samples were additionally annotated to 38 fine cell types (\code{"label.fine"}):
#' \itemize{
#'     \item Basophils
#'     \item Naive B cells
#'     \item Mature B cells class able to switch
#'     \item Mature B cells
#'     \item Mature B cells class switched
#'     \item Common myeloid progenitors
#'     \item Plasmacytoid Dendritic Cells
#'     \item Myeloid Dendritic Cells
#'     \item Eosinophils
#'     \item Erythroid_CD34+ CD71+ GlyA-
#'     \item Erythroid_CD34- CD71+ GlyA-
#'     \item Erythroid_CD34- CD71+ GlyA+
#'     \item Erythroid_CD34- CD71lo GlyA+
#'     \item Erythroid_CD34- CD71- GlyA+
#'     \item Granulocyte/monocyte progenitors
#'     \item Colony Forming Unit-Granulocytes
#'     \item Granulocyte (Neutrophilic Metamyelocytes)
#'     \item Granulocyte (Neutrophils)
#'     \item Hematopoietic stem cells_CD133+ CD34dim
#'     \item Hematopoietic stem cell_CD38- CD34+
#'     \item Colony Forming Unit-Megakaryocytic
#'     \item Megakaryocytes
#'     \item Megakaryocyte/erythroid progenitors
#'     \item Colony Forming Unit-Monocytes
#'     \item Monocytes
#'     \item Mature NK cells_CD56- CD16+ CD3-
#'     \item Mature NK cells_CD56+ CD16+ CD3-
#'     \item Mature NK cells_CD56- CD16- CD3-
#'     \item NK T cells
#'     \item Early B cells
#'     \item Pro B cells
#'     \item CD8+ Effector Memory RA
#'     \item Naive CD8+ T cells
#'     \item CD8+ Effector Memory
#'     \item CD8+ Central Memory
#'     \item Naive CD4+ T cells
#'     \item CD4+ Effector Memory
#'     \item CD4+ Central Memory
#' }
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#' 
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Jared Andrews
#' 
#' @references
#' Novershtern N et al. (2011).
#' Densely interconnected transcriptional circuits control cell states in human hematopoiesis.
#' \emph{Cell} 144, 296-309.
#' 
#' @examples
#' ref.se <- NovershternHematopoieticData()
#' 
#' @export
NovershternHematopoieticData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("novershtern_hematopoietic", "2024-02-26", realize.assays=TRUE)
    } else {
        se <- .create_se("dmap", 
            version = c(logcounts="1.0.0", coldata="1.2.0"), 
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "novershtern", cell.ont)
    }

    .convert_to_ensembl(se, "Hs", ensembl)
}

# Test that all datasets can be loaded with their arguments.
# library(testthat); library(SingleR); source("test-datasets.R")

test_that("Ensembl coercion works for all datasets", {
    expect_error(X <- BlueprintEncodeData(ensembl=TRUE), NA)

    expect_error(X <- DatabaseImmuneCellExpressionData(ensembl=TRUE), NA)

    expect_error(X <- HumanPrimaryCellAtlasData(ensembl=TRUE), NA)

    expect_error(X <- ImmGenData(ensembl=TRUE), NA)

    expect_error(X <- MonacoImmuneData(ensembl=TRUE), NA)

    expect_error(X <- MouseRNAseqData(ensembl=TRUE), NA)

    expect_error(X <- NovershternHematopoieticData(ensembl=TRUE), NA)
})

test_that("ontology mapping works for all datasets", {
    expect_error(X <- BlueprintEncodeData(cell.ont="nonna"), NA)

    expect_error(X <- DatabaseImmuneCellExpressionData(cell.ont="nonna"), NA)

    expect_error(X <- HumanPrimaryCellAtlasData(cell.ont="nonna"), NA)

    expect_error(X <- ImmGenData(cell.ont="nonna"), NA)

    expect_error(X <- MonacoImmuneData(cell.ont="nonna"), NA)

    expect_error(X <- MouseRNAseqData(cell.ont="nonna"), NA)

    expect_error(X <- NovershternHematopoieticData(cell.ont="nonna"), NA)
})

#' Obtain mouse bulk expression data from the Immunologic Genome Project
#'
#' Download and cache the normalized expression values of 830 microarray samples of
#' pure mouse immune cells, generated by the Immunologic Genome Project (ImmGen).
#'
#' @inheritParams HumanPrimaryCellAtlasData
#'
#' @details
#' This function provides normalized expression values of 830 microarray samples
#' generated by ImmGen from pure populations of murine immune cells (<http://www.immgen.org/>).
#' The samples were processed and normalized as described in Aran, Looney and
#' Liu et al. (2019), i.e., CEL files from the Gene Expression Omnibus (GEO; GSE15907 and GSE37448), 
#' were downloaded, processed, and normalized using the robust multi-array average
#' (RMA) procedure on probe-level data.
#' 
#' This dataset consists of 20 broad cell types (\code{"label.main"}) and 253
#' finely resolved cell subtypes (\code{"label.fine"}).
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#'
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Friederike Dündar
#' 
#' @references
#' Heng TS, Painter MW, Immunological Genome Project Consortium (2008).
#' The Immunological Genome Project: networks of gene expression in immune cells.
#' \emph{Nat. Immunol.} 9, 1091-1094. 
#' 
#' Aran D, Looney AP, Liu L et al. (2019). 
#' Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
#' \emph{Nat. Immunol.} 20, 163–172. 
#' 
#' @examples
#' ref.se <- ImmGenData()
#' 
#' @export
ImmGenData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("immgen", "2024-02-26", realize.assays=TRUE)
    } else {
        se <- .create_se("immgen", version = "1.0.0",
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "immgen", cell.ont)
    }

    se <- .convert_to_ensembl(se, "Mm", ensembl)

    se
}

#' Fetch a reference dataset 
#'
#' Fetch a reference dataset (or its metadata) from the gypsum backend.
#'
#' @param name String containing the name of the reference dataset.
#' @param version String containing the version of the dataset.
#' @param path String containing the path to a subdataset, if \code{name} contains multiple reference datasets.
#' Defaults to \code{NA} if no subdatasets are present.
#' @param package String containing the name of the package.
#' @param cache,overwrite Arguments to pass to \code{\link[gypsum]{saveVersion}} or \code{\link[gypsum]{saveFile}}.
#' @param realize.assays Logical scalar indicating whether to realize assays into memory.
#' Dense and sparse \linkS4class{ReloadedArray} objects are converted into ordinary arrays and \linkS4class{dgCMatrix} objects, respectively.
#' @param ... Further arguments to pass to \code{\link{readObject}}.
#'
#' @return \code{fetchReference} returns the dataset as a \linkS4class{SummarizedExperiment}.
#' This is guaranteed to have a \code{"logcounts"} assay with log-normalized expression values,
#' along with at least one character vector of labels in the column data.
#'
#' \code{fetchMetadata} returns a named list of metadata for the specified dataset.
#'
#' @seealso
#' \url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.
#'
#' \code{\link{saveReference}} and \code{\link{uploadDirectory}}, to save and upload a dataset.
#'
#' \code{\link{listReferences}} and \code{\link{listVersions}}, to get possible values for \code{name} and \code{version}.
#' 
#' @author Aaron Lun
#' @examples
#' fetchReference("immgen", "2024-02-26")
#' str(fetchMetadata("immgen", "2024-02-26"))
#'
#' @export
#' @importFrom gypsum cacheDirectory saveVersion
#' @importFrom alabaster.base altReadObjectFunction altReadObject
fetchReference <- function(name, version, path=NA, package="celldex", cache=cacheDirectory(), overwrite=FALSE, realize.assays=FALSE, ...) {
    version_path <- saveVersion(package, name, version, cache=cache, overwrite=overwrite)

    obj_path <- version_path
    if (!is.na(path)) {
        obj_path <- file.path(version_path, gsub("/*$", "", path))
    }

    old <- altReadObjectFunction(cdLoadObject)
    on.exit(altReadObjectFunction(old))
    altReadObject(obj_path, celldex.realize.assays=realize.assays, ...)
}

#' @export
#' @rdname fetchReference
#' @importFrom jsonlite fromJSON
#' @importFrom gypsum cacheDirectory saveFile
fetchMetadata <- function(name, version, path=NA, package="celldex", cache=cacheDirectory(), overwrite=FALSE) {
    remote_path <- "_bioconductor.json"
    if (!is.na(path)) {
        remote_path <- paste0(path, "/", remote_path)
    }

    local_path <- saveFile(package, name, version, remote_path, cache=cache, overwrite=overwrite)
    fromJSON(local_path, simplifyVector=FALSE)
}

#' @import methods
#' @importFrom alabaster.base readObjectFile readObject
#' @importFrom SummarizedExperiment assay assay<- assayNames
cdLoadObject <- function(path, metadata=NULL, celldex.realize.assays=FALSE, ...) {
    if (is.null(metadata)) {
        metadata <- readObjectFile(path)
    }
    ans <- readObject(
        path, 
        metadata=metadata, 
        celldex.realize.assays=celldex.realize.assays, 
        ...
    )

    if (is(ans, "SummarizedExperiment")) {
        if (celldex.realize.assays) {
            for (y in assayNames(ans)) {
                assay(ans, y, withDimnames=FALSE) <- realize_array(assay(ans, y, withDimnames=FALSE))
            }
        }
    }

    ans
}

#' @import methods
#' @importClassesFrom alabaster.matrix ReloadedArray
#' @importFrom DelayedArray is_sparse type
#' @importClassesFrom Matrix lgCMatrix dgCMatrix
realize_array <- function(x) {
    if (is(x, "ReloadedArray")) {
        if (is_sparse(x)) {
            if (type(x) == "logical") {
                x <- as(x, "lgCMatrix")
            } else {
                x <- as(x, "dgCMatrix")
            }
        } else {
            x <- as.array(x)
        }
    }
    x
}

# This tests the fetching of reference datasets.
# library(testthat); library(celldex); source("test-fetchReference.R")

test_that("fetchReference works as expected", {
    sce <- fetchReference("hpca", "2024-02-26")
    expect_s4_class(sce, "SummarizedExperiment")

    # Correctly creates ReloadedMatrix objects.
    ass <- assay(sce, withDimnames=FALSE)
    expect_s4_class(ass, "ReloadedMatrix")
    expect_false(DelayedArray::is_sparse(ass))
    expect_true(grepl("hpca", ass@seed@path))
    expect_true(grepl("2024-02-26", ass@seed@path))

    # Works with realization options.
    sce <- fetchReference("hpca", "2024-02-26", realize.assays=TRUE)
    expect_type(assay(sce, withDimnames=FALSE), "double")
})

test_that("fetchMetadata works as expected", {
    meta <- fetchMetadata("hpca", "2024-02-26")
    expect_match(meta$title, "Human Primary Cell Atlas")
    expect_identical(meta$taxonomy_id[[1]], "9606")
})

#' Obtain bulk RNA-seq data of sorted human immune cells
#'
#' Download and cache the normalized expression values of 114 bulk RNA-seq samples
#' of sorted immune cell populations that can be found in 
#' \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE107011}{GSE107011}.
#'
#' @inheritParams HumanPrimaryCellAtlasData
#'
#' @details 
#' The dataset contains 114 human RNA-seq samples annotated to 10 main cell types (\code{"label.main"}):
#' \itemize{
#'     \item CD8+ T cells
#'     \item T cells
#'     \item CD4+ T cells
#'     \item Progenitors
#'     \item B cells
#'     \item Monocytes
#'     \item NK cells
#'     \item Dendritic cells
#'     \item Neutrophils
#'     \item Basophils
#' }
#'
#' Samples were additionally annotated to 29 fine cell types (\code{"label.fine"}):
#' \itemize{
#'     \item Naive CD8 T cells
#'     \item Central memory CD8 T cells
#'     \item Effector memory CD8 T cells
#'     \item Terminal effector CD8 T cells
#'     \item MAIT cells
#'     \item Vd2 gd T cells
#'     \item Non-Vd2 gd T cells
#'     \item Follicular helper T cells
#'     \item T regulatory cells
#'     \item Th1 cells
#'     \item Th1/Th17 cells
#'     \item Th17 cells
#'     \item Th2 cells
#'     \item Naive CD4 T cells
#'     \item Terminal effector CD4 T cells
#'     \item Progenitor cells
#'     \item Naive B cells
#'     \item Non-switched memory B cells
#'     \item Exhausted B cells
#'     \item Switched memory B cells
#'     \item Plasmablasts
#'     \item Classical monocytes
#'     \item Intermediate monocytes
#'     \item Non classical monocytes
#'     \item Natural killer cells
#'     \item Plasmacytoid dendritic cells
#'     \item Myeloid dendritic cells
#'     \item Low-density neutrophils
#'     \item Low-density basophils
#' }
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#'
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Jared Andrews
#' 
#' @references
#' Monaco G et al. (2019).
#' RNA-Seq Signatures Normalized by mRNA Abundance Allow Absolute Deconvolution of Human Immune Cell Types
#' \emph{Cell Rep.} 26, 1627-1640.
#' 
#' @examples
#' ref.se <- MonacoImmuneData()
#' 
#' @export
MonacoImmuneData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("monaco_immune", "2024-02-26", realize.assays=TRUE)
    } else {
        version <- "1.0.0"
        se <- .create_se("monaco_immune", version, 
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "monaco", cell.ont)
    }

    .convert_to_ensembl(se, "Hs", ensembl)
}

#' Obtain mouse bulk expression data of sorted cell populations (RNA-seq)
#'
#' Download and cache the normalized expression values of 358 bulk RNA-seq samples
#' of sorted cell populations that can be found at GEO.
#'
#' @inheritParams HumanPrimaryCellAtlasData
#'
#' @details This dataset was contributed by the Benayoun Lab that identified, 
#' downloaded and processed data sets on GEO that corresponded to sorted cell
#' types (Benayoun et al., 2019).
#' 
#' The dataset contains 358 mouse RNA-seq samples annotated to 18 main cell types (\code{"label.main"}):
#' \itemize{
#'     \item Adipocytes
#'     \item Astrocytes
#'     \item B cells 
#'     \item Cardiomyocytes
#'     \item Dendritic cells
#'     \item Endothelial cells
#'     \item Epithelial cells
#'     \item Erythrocytes
#'     \item Fibroblasts
#'     \item Granulocytes
#'     \item Hepatocytes
#'     \item Macrophages
#'     \item Microglia
#'     \item Monocytes
#'     \item Neurons
#'     \item NK cells
#'     \item Oligodendrocytes
#'     \item T cells
#' } 
#' These are split further into 28 subtypes (\code{"label.fine"}).
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#'
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Friederike Dündar
#' 
#' @references
#' Benayoun B et al. (2019).
#' Remodeling of epigenome and transcriptome landscapes with aging in mice reveals widespread induction of inflammatory responses.
#' \emph{Genome Res.} 29, 697-709.
#' 
#' Code at \url{https://github.com/BenayounLaboratory/Mouse_Aging_Epigenomics_2018/tree/master/FigureS7_CIBERSORT/RNAseq_datasets_for_Deconvolution/2017-01-18}
#' 
#' @examples
#' ref.se <- MouseRNAseqData()
#' 
#' @export
MouseRNAseqData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("mouse_rnaseq", "2024-02-26", realize.assays=TRUE)
    } else {
        version <- "1.0.0"
        se <- .create_se("mouse.rnaseq", version, 
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "mouse_rnaseq", cell.ont)
    }

    .convert_to_ensembl(se, "Mm", ensembl)
}

#' @importFrom ExperimentHub ExperimentHub
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importFrom SummarizedExperiment rowData
#' @importFrom S4Vectors DataFrame
.create_se <- function(dataset, version, hub = ExperimentHub(), assays="logcounts",
    rm.NA = c("rows","cols","both","none"), has.rowdata=FALSE, has.coldata=TRUE) 
{
    rm.NA <- match.arg(rm.NA)
    host <- file.path("celldex", dataset)

    ## extract normalized values --------
    all.assays <- list()
    for (a in assays) {
        ver <- .fetch_version(version, a)
        nrmcnts <- hub[hub$rdatapath==file.path(host, ver, paste0(a, ".rds"))][[1]]
        all.assays[[a]] <- .rm_NAs(nrmcnts, rm.NA)
    }
    
    ## get metadata ----------------------
    args <- list()
    if (has.coldata) {
        ver <- .fetch_version(version, "coldata")
        args$colData <- hub[hub$rdatapath==file.path(host, ver, "coldata.rds")][[1]]
    }
    if (has.rowdata) {
        ver <- .fetch_version(version, "rowdata")
        args$rowData <- hub[hub$rdatapath==file.path(host, ver, "rowdata.rds")][[1]]
    }
    
    ## make the final SE object ----------
    do.call(SummarizedExperiment, c(list(assays=all.assays), args))
}

.fetch_version <- function(version, field) {
    opt <- version[field]
    if (is.na(opt)) {
        version[1]
    } else {
        opt
    }
}

#' @importFrom DelayedMatrixStats rowAnyNAs colAnyNAs
#' @importFrom DelayedArray DelayedArray
.rm_NAs <- function(mat, rm.NA = "rows"){
    # Identify them first before removal, to ensure that 
    # the same rows are columns are removed with 'both'.
    if(rm.NA == "rows" || rm.NA == "both"){
        keep_rows <- !rowAnyNAs(DelayedArray(mat))
    } else {
        keep_rows <- !logical(nrow(mat))
    }

    if (rm.NA=="cols" || rm.NA == "both") {
        keep_cols <- !colAnyNAs(DelayedArray(mat))
    } else {
        keep_cols <- !logical(ncol(mat))
    }

    # Avoid making unnecessary copies if possible.
    if (!all(keep_rows)) {
        mat <- mat[keep_rows,,drop=FALSE]
    }
    if (!all(keep_cols)) {
        mat <- mat[,keep_cols,drop=FALSE]
    }
    mat
}

#' @importFrom AnnotationHub AnnotationHub
#' @importFrom AnnotationDbi mapIds
.convert_to_ensembl <- function(se, species, ensembl) 
# Copied verbatim from scRNAseq... well, whatever.
{
    if (ensembl) {
        if (species=="Mm") {
            tag <- "AH73905"
        } else if (species=="Hs") {
            tag <- "AH73881"
        }

        edb <- AnnotationHub()[[tag]]
        ensid <- mapIds(edb, keys=rownames(se), keytype="SYMBOL", column="GENEID")

        keep <- !is.na(ensid) & !duplicated(ensid)
        if (!all(keep)) {
            se <- se[keep,]
        }
        rownames(se) <- ensid[keep]
    }

    se
}

#' @importFrom utils read.delim
.add_ontology <- function(se, fname, mode) {
    if (mode!="none") {
        path <- system.file("mapping", paste0(fname, ".tsv"), package="celldex", mustWork=TRUE)
        src <- read.delim(path, header=FALSE, stringsAsFactors=FALSE)

        m <- match(se$label.fine, src[,1])
        stopifnot(all(!is.na(m))) # sanity check

        matched <- src[m, 2]
        matched[matched==""] <- NA_character_
        se$label.ont <- matched

        if (mode=="nonna") {
            se <- se[,!is.na(se$label.ont)]
        }
    }

    se
}

#' Survey reference metadata
#'
#' Metadata survey for all available reference datasets in the \pkg{celldex} package.
#'
#' @param cache,overwrite Arguments to pass to \code{\link{fetchMetadataDatabase}}.
#' @param latest Whether to only consider the latest version of each dataset.
#'
#' @return 
#' A \linkS4class{DataFrame} where each row corresponds to a dataset, containing various columns of metadata.
#' Some columns may be lists to capture 1:many mappings.
#'
#' @details
#' The returned DataFrame contains the usual suspects like the title and description for each dataset,
#' the number of samples and types of labels, the organisms and genome builds involved, and so on.
#' More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 
#'
#' @author Aaron Lun
#'
#' @examples
#' surveyReferences()
#'
#' @seealso
#' \code{\link{searchReferences}}, to search on the metadata for specific datasets.
#' 
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom gypsum cacheDirectory fetchMetadataDatabase
#' @importFrom DBI dbConnect dbDisconnect dbGetQuery
#' @importFrom RSQLite SQLite
surveyReferences <- function(cache=cacheDirectory(), overwrite=FALSE, latest=TRUE) {
    bpath <- fetchMetadataDatabase(cache=cache, overwrite=overwrite)
    con <- dbConnect(SQLite(), bpath)
    on.exit(dbDisconnect(con))

    stmt <- "SELECT json_extract(metadata, '$') AS meta, versions.asset AS asset, versions.version AS version, path";
    if (!latest) {
        stmt <- paste0(stmt, ", versions.latest AS latest")
    }
    stmt <- paste0(stmt, " FROM paths LEFT JOIN versions ON paths.vid = versions.vid WHERE versions.project = 'celldex'")
    if (latest) {
        stmt <- paste0(stmt, " AND versions.latest = 1")
    }
    everything <- dbGetQuery(con, stmt)

    sanitize_query_to_output(everything, latest)
}

#' @import methods
#' @importFrom S4Vectors DataFrame
#' @importFrom jsonlite fromJSON
sanitize_query_to_output <- function(results, latest, meta.name="meta") {
    path <- results$path
    has.slash <- grepl("/", path)
    path[!has.slash] <- NA_character_
    path[has.slash] <- sub("/[^/]+$", "", path[has.slash])
    df <- DataFrame(name = results$asset, version = results$version, path = path)
    if (!latest) {
        df$latest <- results$latest == 1
    }

    all_meta <- lapply(results[[meta.name]], fromJSON, simplifyVector=FALSE)
    df$title <- extract_atomic_from_json(all_meta, function(x) x$title, "character")
    df$description <- extract_atomic_from_json(all_meta, function(x) x$title, "character")
    df$taxonomy_id <- extract_charlist_from_json(all_meta, function(x) x$taxonomy_id)
    df$genome <- extract_charlist_from_json(all_meta, function(x) x$genome)

    df$samples <- extract_atomic_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$columns, "integer")
    df$labels <- extract_charlist_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$column_annotations)

    df$bioconductor_version < extract_atomic_from_json(all_meta, function(x) x$bioconductor_version, "character")
    df$maintainer_name < extract_atomic_from_json(all_meta, function(x) x$maintainer_name, "character")
    df$maintainer_email < extract_atomic_from_json(all_meta, function(x) x$maintainer_email, "character")

    sources <- vector("list", length(all_meta))
    for (i in seq_along(all_meta)) {
        cursources <- all_meta[[i]]$sources
        if (is.null(cursources)) {
            sources[[i]] <- DataFrame(provider=character(0), id=character(0), version=character(0))
        } else {
            sources[[i]] <- DataFrame(
                provider = extract_atomic_from_json(cursources, function(x) x$provider, "character"),
                id = extract_atomic_from_json(cursources, function(x) x$id, "character"),
                version = extract_atomic_from_json(cursources, function(x) x$version, "character")
            )
        }
    }
    df$sources <- as(sources, "CompressedList")

    df
}

extract_atomic_from_json <- function(metadata, extract, type) {
    vapply(metadata, function(y) {
        x <- extract(y)
        if (is.null(x)) {
            as(NA, type)
        } else {
            x
        }
    }, vector(type, 1))
}

#' @import methods
extract_charlist_from_json <- function(metadata, extract) {
    output <- lapply(metadata, function(y) {
        x <- extract(y)
        if (is.null(y)) {
            character(0)
        } else {
            x
        }
    })
    as(output, "CompressedList")
}

#' Save a reference dataset 
#'
#' Save a reference dataset to disk, usually in preparation for upload via \code{\link{uploadDirectory}}.
#'
#' @param x Matrix of log-normalized expression values.
#' This may be sparse or dense, but should be non-integer and have no missing values.
#' Row names should be present and unique for all rows.
#' @param labels \linkS4class{DataFrame} of labels.
#' Each row of \code{labels} corresponds to a column of \code{x} and contains the label(s) for that column.
#' Each column of \code{labels} represents a different label type;
#' typically, the column name has a \code{label.} prefix to distinguish between, e.g., \code{label.fine}, \code{label.broad} and so on.
#' At least one column should be present.
#' @param path String containing the path to a directory in which to save \code{x}.
#' @param metadata Named list containing metadata for this reference dataset,
#' see the schema returned by \code{\link{fetchMetadataSchema}()}.
#' Note that the \code{applications.takane} property will be automatically added by this function and does not have to be supplied.
#'
#' @return \code{x} and \code{labels} are used to create a \linkS4class{SummarizedExperiment} that is saved into \code{path}.
#' \code{NULL} is invisibly returned.
#'
#' @details
#' The SummarizedExperiment saved to \code{path} is guaranteed to have the \code{"logcounts"} assay and at least one column in \code{labels}.
#' This mirrors the expectation for reference datasets obtained via \code{\link{fetchReference}}.
#'
#' @author Aaron Lun
#' @examples
#' # Mocking up some data to be saved.
#' x <- matrix(rnorm(1000), ncol=10)
#' rownames(x) <- sprintf("GENE_%i", seq_len(nrow(x)))
#' labels <- DataFrame(
#'     labels.broad = sample(c("B", "T", "NK"), ncol(x), replace=TRUE),
#'     labels.fine = sample(c("PC", "pre-B", "pro-B", "Th2", "CD4+T", "NK"), 
#'         ncol(x), replace=TRUE)
#' )
#'
#' # Making up some metadata as well.
#' meta <- list(
#'     title="New reference dataset",
#'     description="This is a new reference dataset, generated from blah blah.",
#'     genome="GRCm38",
#'     taxonomy_id="10090",
#'     sources=list(
#'         list(provider="GEO", id="GSE123456"),
#'         list(provider="PubMed", id="123456"),
#'         list(provider="URL", id="https://reference.data.com", version="2024-02-26")
#'     ),
#'     maintainer_name="Chihaya Kisaragi",
#'     maintainer_email="kisaragi.chihaya@765.com"
#' )
#' 
#' # Actually saving it.
#' tmp <- tempfile()
#' saveReference(x, labels, tmp, meta)
#'
#' # Reloading it to make sure it looks good.
#' alabaster.base::readObject(tmp)
#' str(jsonlite::fromJSON(file.path(tmp, "_bioconductor.json")))
#'
#' @seealso
#' \code{\link{uploadDirectory}}, to upload the saved dataset to the gypsum backend.
#'
#' \code{\link{fetchReference}}, to download an existing dataset into the current sesssion.
#'
#' @export
#' @importFrom alabaster.base saveObject
#' @importMethodsFrom alabaster.se saveObject
#' @importFrom DelayedArray type
#' @importFrom SummarizedExperiment SummarizedExperiment assay<- assayNames colData
#' @importFrom gypsum fetchMetadataSchema validateMetadata formatObjectMetadata
#' @importFrom jsonlite toJSON 
saveReference <- function(x, labels, path, metadata) {
    schema <- fetchMetadataSchema()
    if (is.null(metadata$bioconductor_version)) {
        metadata$bioconductor_version <- as.character(BiocManager::version())
    }
    metadata$taxonomy_id <- I(metadata$taxonomy_id)
    metadata$genome <- I(metadata$genome)
    validateMetadata(metadata, schema) # First validation for user-supplied content.

    # Check that all labels are categorical.
    if (ncol(labels) == 0L) {
        stop("'labels' should contain at least one column")
    }
    for (cn in colnames(labels)) {
        if (!is.character(labels[[cn]])) {
            stop("all columns of 'labels' should be character vectors")
        }
    }

    # Checking the assay data.
    if (type(x) != "double" || all(x %% 1 == 0)) {
        stop("'x' should contain log-normalized expression values")
    }
    if (anyNA(x)) {
        stop("detected missing values in 'x'")
    }
    rnms  <- rownames(x)
    if (is.null(rnms) && anyDuplicated(rnms)) {
        stop("'x' should have unique row names")
    }

    se <- SummarizedExperiment(list(logcounts=x), colData=labels)
    assay(se, withDimnames=FALSE) <- unname(x)
    unlink(path, recursive=TRUE)
    saveObject(se, path, ReloadedArray.reuse.files="symlink")

    # Attach some takane metadata to it.
    takane <- formatObjectMetadata(se)
    takane$type <- readObjectFile(path)$type
    if (is.null(metadata$applications)) {
        metadata$applications <- list()
    }
    metadata$applications$takane <- takane

    # Second validation with the takane metadata.
    contents <- toJSON(metadata, pretty=4, auto_unbox=TRUE)
    validateMetadata(contents, schema=schema)
    write(contents, file=file.path(path, "_bioconductor.json"))
    invisible(NULL)
}

# This checks that saveReference works as expected.
# library(testthat); library(celldex); source("test-saveReference.R")

# Mocking up some data to be saved.
x <- matrix(rnorm(1000), ncol=10)
rownames(x) <- sprintf("GENE_%i", seq_len(nrow(x)))
labels <- DataFrame(
    labels.broad = sample(c("B", "T", "NK"), ncol(x), replace=TRUE),
    labels.fine = sample(c("PC", "pre-B", "pro-B", "Th2", "CD4+T", "NK"), 
        ncol(x), replace=TRUE)
)
                                                                                    
test_that("saveReference works as expected", {
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    tmp <- tempfile()
    saveReference(x, labels, tmp, meta)
                                                                                        
    se <- alabaster.base::readObject(tmp)
    expect_s4_class(se, "SummarizedExperiment")
    meta <- jsonlite::fromJSON(file.path(tmp, "_bioconductor.json"), simplifyVector=FALSE)
    expect_identical(meta$bioconductor_version, as.character(BiocManager::version()))

    # Validation fails as expected.
    tmp <- tempfile()
    meta$title <- 1234
    expect_error(saveReference(x, labels, tmp, meta), "title")
})

test_that("saveReference works with ReloadedArray objects", {
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    # Saving something to link to.
    tmp <- tempfile()
    saveReference(x, labels, tmp, meta)

    # Adding a ReloadedArray assay.
    x2 <- alabaster.matrix::ReloadedArray(path=file.path(tmp, "assays", "0"), seed=matrix(runif(1000), 100, 10))
    rownames(x2) <- rownames(x)

    # Checking that the link targets are created correctly.
    tmp2 <- tempfile()
    saveReference(x2, labels, tmp2, meta)
    expect_true(nchar(Sys.readlink(file.path(tmp2, "assays", "0", "array.h5"))) > 0L) # check that these are indeed symlinks.

    roundtrip <- alabaster.base::readObject(tmp2)
    expect_identical(as.matrix(assay(roundtrip)), x) # all assays are just symlinked to the first save!
})

#' Obtain human bulk RNA-seq data from Blueprint and ENCODE
#'
#' Download and cache the normalized expression values of 259 RNA-seq samples of
#' pure stroma and immune cells as generated and supplied by Blueprint and ENCODE.
#'
#' @inheritParams HumanPrimaryCellAtlasData
#' @param rm.NA String specifying how missing values should be handled.
#' \code{"rows"} will remove genes with at least one missing value, 
#' \code{"cols"} will remove samples with at least one missing value,
#' \code{"both"} will remove any gene or sample with at least one missing value,
#' and \code{"none"} will not perform any removal.
#'
#' @details
#' This function provides normalized expression values for 259 bulk RNA-seq samples
#' generated by Blueprint and ENCODE from pure populations of stroma and immune 
#' cells (Martens and Stunnenberg, 2013; The ENCODE Consortium, 2012).
#' The samples were processed and normalized as described in Aran, Looney and
#' Liu et al. (2019), i.e., the raw RNA-seq counts were downloaded from 
#' Blueprint and ENCODE in 2016 and normalized via edgeR (TPMs).
#' 
#' Blueprint Epigenomics contains 144 RNA-seq pure immune samples annotated to 28 cell types.
#' ENCODE contains 115 RNA-seq pure stroma and immune samples annotated to 17 cell types.
#' All together, this reference contains 259 samples with 43 cell types (\code{"label.fine"}),
#' manually aggregated into 24 broad classes (\code{"label.main"}).
#' The fine labels have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#' 
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Friederike Dündar
#'
#' @references
#' The ENCODE Project Consortium (2012).
#' An integrated encyclopedia of DNA elements in the human genome.
#' \emph{Nature} 489, pages 57–74.
#' 
#' Martens JHA and Stunnenberg HG (2013). 
#' BLUEPRINT: mapping human blood cell epigenomes.
#' \emph{Haematologica} 98, 1487–1489.
#' 
#' Aran D, Looney AP, Liu L et al. (2019). 
#' Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
#' \emph{Nat. Immunol.} 20, 163–172. 
#' 
#' @examples
#' ref.se <- BlueprintEncodeData(rm.NA = "rows")
#'
#' @export
BlueprintEncodeData <- function(rm.NA = c("rows","cols","both","none"), ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    rm.NA <- match.arg(rm.NA)
    cell.ont <- match.arg(cell.ont)

    if (!legacy && rm.NA == "rows" && cell.ont == "all") {
        se <- fetchReference("blueprint_encode", "2024-02-26", realize.assays=TRUE)
    } else {
        se <- .create_se("blueprint_encode", 
            version = list(logcounts="1.0.0", coldata="1.2.0"),
            assays="logcounts", rm.NA = rm.NA,
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "blueprint_encode", cell.ont)
    }

    .convert_to_ensembl(se, "Hs", ensembl)
}

#' Obtain human bulk RNA-seq data from DICE
#'
#' Download and cache the normalized expression values of 1561 bulk RNA-seq samples
#' of sorted cell populations from the Database of Immune Cell Expression (DICE).
#'
#' @inheritParams HumanPrimaryCellAtlasData
#'
#' @details 
#' This function provides normalized expression values of 1561 bulk RNA-seq samples
#' generated by DICE from pure populations of human immune cells.
#'
#' TPM normalized values for each cell type were downloaded from \url{https://dice-database.org/downloads}. 
#' Genes with no reads across samples were removed, and values were log2 normalized after a pseudocount of 1 was added.
#' 
#' The dataset contains 1561 human RNA-seq samples annotated to 5 main cell types (\code{"label.main"}):
#' \itemize{
#'     \item B cells
#'     \item Monocytes
#'     \item NK cells
#'     \item T cells, CD8+
#'     \item T cells, CD4+
#' }
#'
#' Samples were additionally annotated to 15 fine cell types (\code{"label.fine"}):
#' \itemize{
#'     \item B cells, naive
#'     \item Monocytes, CD14+
#'     \item Monocytes, CD16+
#'     \item NK cells
#'     \item T cells, memory TREG
#'     \item T cells, CD4+, naive
#'     \item T cells, CD4+, naive, stimulated
#'     \item T cells, CD4+, naive Treg
#'     \item T cells, CD4+, Th1
#'     \item T cells, CD4+, Th1_17
#'     \item T cells, CD4+, Th2
#'     \item T cells, CD8+, naïve
#'     \item T cells, CD8+, naïve, stimulated
#'     \item T cells, CD4+, TFH
#'     \item T cells, CD4+, Th17
#' }
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#'
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Jared Andrews
#' 
#' @references
#' Schmiedel B et al. (2018).
#' Impact of Genetic Polymorphisms on Human Immune Cell Gene Expression.
#' \emph{Cell} 175, 1701-1715.
#' 
#' @examples
#' ref.se <- DatabaseImmuneCellExpressionData()
#' 
#' @export
DatabaseImmuneCellExpressionData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("dice", "2024-02-26", realize.assays=TRUE)
    } else {
        version <- "1.0.0"
        se <- .create_se("dice", version, 
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "dice", match.arg(cell.ont))
    }

    .convert_to_ensembl(se, "Hs", ensembl)
}

#' Search reference metadata
#'
#' Search for reference datasets of interest based on matching text in the associated metadata.
#'
#' @param query String or a \code{gypsum.search.object}, see Examples.
#' @inheritParams surveyReferences
#'
#' @return 
#' A \linkS4class{DataFrame} where each row corresponds to a dataset, containing various columns of metadata.
#' Some columns may be lists to capture 1:many mappings.
#'
#' @details
#' The returned DataFrame contains the usual suspects like the title and description for each dataset,
#' the number of rows and columns, the organisms and genome builds involved,
#' whether the dataset has any pre-computed reduced dimensions, and so on.
#' More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 
#'
#' @author Aaron Lun
#'
#' @examples
#' searchReferences(defineTextQuery("immun%", partial=TRUE))[,c("name", "title")]
#' searchReferences(defineTextQuery("10090", field="taxonomy_id"))[,c("name", "title")]
#' searchReferences(
#'    defineTextQuery("10090", field="taxonomy_id") &
#'    defineTextQuery("immun%", partial=TRUE)
#' )[,c("name", "title")]
#' 
#' @seealso
#' \code{\link{surveyReferences}}, to easily obtain a listing of all available datasets.
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom gypsum cacheDirectory fetchMetadataDatabase searchMetadataTextFilter
#' @importFrom DBI dbConnect dbDisconnect dbGetQuery
#' @importFrom RSQLite SQLite
searchReferences <- function(query, cache=cacheDirectory(), overwrite=FALSE, latest=TRUE) {
    filter <- searchMetadataTextFilter(query)

    bpath <- fetchMetadataDatabase(cache=cache, overwrite=overwrite)
    con <- dbConnect(SQLite(), bpath)
    on.exit(dbDisconnect(con))

    stmt <- "SELECT json_extract(metadata, '$') AS meta, versions.asset AS asset, versions.version AS version, path";
    if (!latest) {
        stmt <- paste0(stmt, ", versions.latest AS latest")
    }
    stmt <- paste0(stmt, " FROM paths LEFT JOIN versions ON paths.vid = versions.vid WHERE versions.project = 'celldex'")
    if (latest) {
        stmt <- paste0(stmt, " AND versions.latest = 1")
    }
    if (!is.null(filter)) {
        stmt <- paste0(stmt, " AND ", filter$where)
        everything <- dbGetQuery(con, stmt, params=filter$parameters)
    } else {
        everything <- dbGetQuery(con, stmt)
    }

    sanitize_query_to_output(everything, latest)
}

#' @export
#' @importFrom gypsum defineTextQuery
gypsum::defineTextQuery

#' Obtain the HPCA data
#'
#' Download and cache the normalized expression values of the data stored in
#' the Human Primary Cell Atlas. The data will be downloaded from ExperimentHub,
#' returning a \linkS4class{SummarizedExperiment} object for further use.
#'
#' @param ensembl Logical scalar indicating whether to convert row names to Ensembl IDs.
#' Genes without a mapping to a non-duplicated Ensembl ID are discarded.
#' @param cell.ont String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
#' If \code{"nonna"}, all samples without a valid term are discarded;
#' if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
#' if \code{"none"}, terms are not added.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' This function provides normalized expression values for 713 microarray samples from
#' the Human Primary Cell Atlas (HPCA) (Mabbott et al., 2013).
#' These 713 samples were processed and normalized as described in Aran, Looney and 
#' Liu et al. (2019).
#' 
#' Each sample has been assigned to one of 37 main cell types (\code{"label.main"}) 
#' and 157 subtypes (\code{"label.fine"}).
#' The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"}, 
#' if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
#' queries.
#'
#' @return A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
#' containing the log-normalized expression values, along with cell type labels in the 
#' \code{\link{colData}}.
#'
#' @author Friederike Dündar
#'
#' @references
#' Mabbott NA et al. (2013).
#' An expression atlas of human primary cells: inference of gene function from coexpression networks.
#' \emph{BMC Genomics} 14, Article 632. 
#' 
#' Aran D, Looney AP, Liu L et al. (2019). 
#' Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
#' \emph{Nat. Immunol.} 20, 163–172. 
#' 
#' @examples
#' ref.se <- HumanPrimaryCellAtlasData()
#'
#' @export
#' @importFrom SummarizedExperiment rowData
HumanPrimaryCellAtlasData <- function(ensembl=FALSE, cell.ont=c("all", "nonna", "none"), legacy=FALSE) {
    cell.ont <- match.arg(cell.ont)

    if (!legacy && cell.ont == "all") {
        se <- fetchReference("hpca", "2024-02-26", realize.assays=TRUE)
    } else {
        version <- "1.0.0"
        se <- .create_se("hpca", version,
            assays="logcounts", rm.NA = "none",
            has.rowdata = FALSE, has.coldata = TRUE)
        se <- .add_ontology(se, "hpca", cell.ont)
    }

    .convert_to_ensembl(se, "Hs", ensembl)
}

#' List available references
#'
#' List the available reference datasets and the associated versions in \pkg{celldex}.
#'
#' @param name String containing the name of the reference dataset.
#'
#' @return 
#' For \code{listReferences}, a character vector containing the names of the available references.
#'
#' For \code{listVersions}, a character vector containing the names of the available versions of the \code{name} reference.
#'
#' For \code{fetchLatestVersion}, a string containing the name of the latest version.
#'
#' @author Aaron Lun
#'
#' @examples
#' listReferences()
#' listVersions("immgen")
#' fetchLatestVersion("immgen")
#'
#' @export
#' @importFrom gypsum listAssets
listReferences <- function() {
    listAssets("celldex")
}

#' @export
#' @rdname listReferences
listVersions <- function(name) {
    gypsum::listVersions("celldex", name)
}

#' @export
#' @rdname listReferences
#' @importFrom gypsum fetchLatest
fetchLatestVersion <- function(name) {
    fetchLatest("celldex", name)
}

# This tests the fetching of reference datasets.
# library(testthat); library(celldex); source("test-listReferences.R")

test_that("listReferences works as expected", {
    refs <- listReferences()
    expect_true("hpca" %in% refs)
    expect_true("immgen" %in% refs)
    expect_true("monaco_immune" %in% refs)
})

test_that("listVersions works as expected", {
    versions <- listVersions("immgen")
    expect_true("2024-02-26" %in% versions)
    latest <- fetchLatestVersion("immgen")
    expect_true(latest %in% versions)
})

---
title: Pokédex for Cell Types
author:
- name: Aaron Lun
  email: infinite.monkeys.with.keyboards@gmail.com
- name: Jared M. Andrews
  affiliation: Washington University in St. Louis, School of Medicine, St. Louis, MO, USA
- name: Friederike Dündar
  affiliation: Applied Bioinformatics Core, Weill Cornell Medicine
- name: Daniel Bunis
  affiliation: Bakar Computational Health Sciences Institute, University of California San Francisco, San Francisco, CA
date: "Revised: February 29, 2024"
output:
  BiocStyle::html_document:
    toc_float: true
package: celldex
bibliography: ref.bib
vignette: >
  %\VignetteIndexEntry{Cell type references}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, results="hide", message=FALSE}
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
library(BiocStyle)
```

# Overview

The `r Biocpkg("celldex")` package provides convenient access to several cell type reference datasets. 
Most of these references are derived from bulk RNA-seq or microarray data of cell populations
that (hopefully) consist of a pure cell type after sorting and/or culturing.
The aim is to provide a common resource for further analysis like cell type annotation of single cell expression data
or deconvolution of cell type proportions in bulk expression datasets.

Each dataset contains a log-normalized expression matrix that is intended to be comparable 
to log-UMI counts from common single-cell protocols [@aran2019reference]
or gene length-adjusted values from bulk datasets.
By default, gene annotation is returned in terms of gene symbols,
but they can be coerced to Ensembl annotation with `ensembl=TRUE` for more robust cross-referencing across studies.

Typically, each reference provides three levels of cell type annotation in its column metadata:

- `label.main`, broad annotation that defines the major cell types.
This has few unique levels that allows for fast annotation but at low resolution.
- `label.fine`, fine-grained annotation that defines subtypes or states.
This has more unique levels that results in slower annotation but at much higher resolution.
- `label.ont`, fine-grained annotation mapped to the standard vocabulary in the [Cell Ontology](https://www.ebi.ac.uk/ols/ontologies/cl).
This enables synchronization of labels across references as well as dynamic adjustment of the resolution.

# Finding references

We can examine the available references using the `surveyReferences()` function.
This returns a `DataFrame` of the reference's name and version, 
along with additional information like the title, description, species, number of samples, available labels, and so on.

```{r}
library(celldex)
surveyReferences()
```

Alternatively, users can search the text of each reference's metadata to identify relevant datasets.
This may require some experimentation as it depends on the level of detail in the metadata supplied by the uploader.

```{r}
searchReferences("B cell")
searchReferences(
    defineTextQuery("immun%", partial=TRUE) & 
    defineTextQuery("10090", field="taxonomy_id")
)
```

Keep in mind that the search results are not guaranteed to be reproducible -
more datasets may be added over time, and existing datasets may be updated with new versions.
Once a dataset of interest is identified, users should explicitly list the name and version of the dataset in their scripts to ensure reproducibility.

# General-purpose references

## Human primary cell atlas (HPCA)

The HPCA reference consists of publicly available microarray datasets derived from human primary cells [@hpcaRef].
Most of the labels refer to blood subpopulations but cell types from other tissues are also available.

```{r}
library(celldex)
ref <- fetchReference("hpca", "2024-02-26")
```

<hr>

```{r tabulate, echo=FALSE}
samples <- colData(ref)[,c("label.main", "label.fine","label.ont")]
samples <- as.data.frame(samples)
DT::datatable(unique(samples))
```

This reference also contains many cells and cell lines that have been treated or collected from pathogenic conditions.

## Blueprint/ENCODE

The Blueprint/ENCODE reference consists of bulk RNA-seq data for pure stroma and immune cells 
generated by Blueprint [@blueprintRef] and ENCODE projects [@encodeRef].

```{r}
ref <- fetchReference("blueprint_encode", "2024-02-26")
```

<hr>

```{r, echo=FALSE, ref.label="tabulate"}
```

This reference is best suited to mixed samples that do not require fine resolution,
and is particularly suited for situations where easily interpretable labels are required quickly. 
It provides decent immune cell granularity, though it does not contain finer monocyte and dendritic cell subtypes.

## Mouse RNA-seq

This reference consists of a collection of mouse bulk RNA-seq data sets downloaded from the gene expression omnibus [@Benayoun2019].
A variety of cell types are available, again mostly from blood but also covering several other tissues.

```{r}
ref <- fetchReference("mouse_rnaseq", "2024-02-26")
```

<hr>

```{r, echo=FALSE, ref.label="tabulate"}
```

This reference is best suited to bulk tissue samples from brain, blood, or heart where low-resolution labels are adequate.

# Immune references

## Immunological Genome Project (ImmGen)

The ImmGen reference consists of microarray profiles of pure mouse immune cells from 
the [project of the same name](http://www.immgen.org/) [@ImmGenRef].
This is currently the most highly resolved immune reference -
possibly overwhelmingly so, given the granularity of the fine labels.

```{r}
ref <- fetchReference("immgen", "2024-02-26")
```

<hr>

```{r, echo=FALSE, ref.label="tabulate"}
```

This reference provides exhaustive coverage of a dizzying number of cell subtypes.
However, this can be a double-edged sword as the high resolution can be difficult to interpret,
especially for samples derived from experimental conditions that are not of interest.
Users may want to remove certain samples themselves depending on the use case. 

## Database of Immune Cell Expression/eQTLs/Epigenomics (DICE)

The DICE reference consists of bulk RNA-seq samples of sorted cell populations
from the [project of the same name](https://dice-database.org) [@diceRef].

```{r}
ref <- fetchReference("dice", "2024-02-26")
```

<hr>

```{r, echo=FALSE, ref.label="tabulate"}
```

This reference is particularly useful to those interested in CD4^+^ T cell subsets,
though the lack of CD4^+^ central memory and effector memory samples may decrease accuracy in some cases. 
In addition, the lack of dendritic cells and a single B cell subset may result in those populations being improperly labeled or having their label pruned in a typical PBMC sample. 

## Novershtern hematopoietic data

The Novershtern reference (previously known as Differentiation Map)
consists of microarray datasets for sorted hematopoietic cell populations
from [GSE24759](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE24759) [@dmapRef].

```{r}
ref <- fetchReference("novershtern_hematopoietic", "2024-02-26")
```

```{r, echo=FALSE, ref.label="tabulate"}
```

This reference provides the greatest resolution for myeloid and progenitor cells among the human immune references. 
It has fewer T cell subsets than the other immune references but contains many more NK, erythroid, and granulocytic subsets. 
It is likely the best option for bone marrow samples.

## Monaco immune data

The Monaco reference consists of bulk RNA-seq samples of sorted immune cell populations 
from [GSE107011](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE107011) [@monaco_immuneRef].

```{r}
ref <- fetchReference("monaco_immune", "2024-02-26")
```

```{r, echo=FALSE, ref.label="tabulate"}
```

This is the human immune reference that best covers all of the bases for a typical PBMC sample. 
It provides expansive B and T cell subsets, differentiates between classical and non-classical monocytes, includes basic dendritic cell subsets, and also includes neutrophil and basophil samples to help identify small contaminating populations that may have slipped into a PBMC preparation.

# Adding new references

Want to contribute your own reference dataset to this package?
It's easy!
Just follow these simple steps for instant fame and prestige. 

1. Obtain log-normalized expression values and cell type labels.
   While not strictly required, we recommend trying to organize the labels into `label.fine`, `label.main` and `label.ont`,
   so that users have a consistent experience with your reference.
   Let's just make up something here.

    ```{r}
    norm <- matrix(runif(1000), ncol=20)
    rownames(norm) <- sprintf("GENE_%i", seq_len(nrow(norm)))

    labels <- DataFrame(label.main=rep(LETTERS[1:5], each=4))
    labels$label.fine <- sprintf("%s%i", labels$label.main, rep(c(1, 1, 2, 2), 5))
    labels$label.ont <- sprintf("CL:000%i", rep(1:5, each=4))
    ```

2. Assemble the metadata for your dataset.
   This should be a list structured as specified in the [Bioconductor metadata schema](https://artifactdb.github.io/bioconductor-metadata-index/bioconductor/v1.json).
   Check out some examples from `fetchMetadata()` - note that the `application.takane` property will be automatically added later, and so can be omitted from the list that you create.

    ```{r}
    meta <- list(
        title="My reference",
        description="This is my reference dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(
            list(provider="GEO", id="GSE12345"),
            list(provider="PubMed", id="1234567")
        ),
        maintainer_name="Chihaya Kisaragi",
        maintainer_email="kisaragi.chihaya@765pro.com"
    )
    ```

3. Save your normalized expression matrix and labels to disk with `saveReference()`.
   This saves the dataset into a "staging directory" using language-agnostic file formats - check out the [**alabaster**](https://github.com/ArtifactDB/alabaster.base) framework for more details.
   In more complex cases involving multiple datasets, users may save each dataset into a subdirectory of the staging directory.

    ```{r}
    # Simple case: you only have one dataset to upload.
    staging <- tempfile()
    saveReference(norm, labels, staging, meta)
    list.files(staging, recursive=TRUE)
    
    # Complex case: you have multiple subdatasets to upload.
    staging <- tempfile()
    dir.create(staging)
    saveReference(norm, labels, file.path(staging, "foo"), meta)
    saveReference(norm, labels, file.path(staging, "bar"), meta) # and so on.
    ```

   You can check that everything was correctly saved by reloading the on-disk data into the R session for inspection.
   This yields a `SummarizedExperiment` with the log-expression matrix in the assay named `"logcounts"`.

    ```{r}
    alabaster.base::readObject(file.path(staging, "foo"))
    ```

4. Open a [pull request (PR)](https://github.com/LTLA/scRNAseq/pulls) for the addition of a new reference.
   You will need to provide a few things here:
   - The name of your dataset.
     This should be short enough to type yet distinct from the existing references.
   - The version of your dataset.
     This is usually just the current date... or whenever you started putting together the dataset for upload.
     The exact date doesn't really matter as long as we can establish a timeline for later versions.
   - The code used to assemble the reference dataset as an Rmarkdown file.
     This should be added to the [`scripts/`](https://github.com/LTLA/celldex/tree/master/scripts) directory of this package,
     in order to provide some record of how the dataset was created.

5. Wait for us to grant temporary upload permissions to your GitHub account.
6. Upload your staging directory to [**gypsum** backend](https://github.com/ArtifactDB/gypsum-worker) with `gypsum::uploadDirectory()`.
   On the first call to this function, it will automatically prompt you to log into GitHub so that the backend can authenticate you.
   If you are on a system without browser access (e.g., most computing clusters), a [token](https://github.com/settings/tokens) can be manually supplied via `gypsum::setAccessToken()`.

    ```{r, eval=FALSE}
    gypsum::uploadDirectory(staging, "celldex", "my_dataset_name", "my_version")
    ```

   You can check that everything was successfully uploaded by just calling `fetchReference()`:

    ```{r, eval=FALSE}
    fetchReference("my_dataset_name", "my_version")
    ```

   If you realized you made a mistake, no worries.
   Use the following call to clear the erroneous dataset, and try again:

    ```{r, eval=FALSE}
    gypsum::rejectProbation("scRNAseq", "my_dataset_name", "my_version")
    ```

7. Comment on the PR to notify us that the dataset has finished uploading and you're happy with it.
   We'll review it and make sure everything's in order.
   If some fixes are required, we'll just clear the dataset so that you can upload a new version with the necessary changes.
   Otherwise, we'll approve the dataset.
   Note that once a version of a dataset is approved, no further changes can be made to that version;
   you'll have to upload a new version if you want to modify something.

# Session information {-}

```{r}
sessionInfo()
```

# References {-}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchReferences.R
\name{searchReferences}
\alias{searchReferences}
\title{Search reference metadata}
\usage{
searchReferences(
  query,
  cache = cacheDirectory(),
  overwrite = FALSE,
  latest = TRUE
)
}
\arguments{
\item{query}{String or a \code{gypsum.search.object}, see Examples.}

\item{cache, overwrite}{Arguments to pass to \code{\link{fetchMetadataDatabase}}.}

\item{latest}{Whether to only consider the latest version of each dataset.}
}
\value{
A \linkS4class{DataFrame} where each row corresponds to a dataset, containing various columns of metadata.
Some columns may be lists to capture 1:many mappings.
}
\description{
Search for reference datasets of interest based on matching text in the associated metadata.
}
\details{
The returned DataFrame contains the usual suspects like the title and description for each dataset,
the number of rows and columns, the organisms and genome builds involved,
whether the dataset has any pre-computed reduced dimensions, and so on.
More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}.
}
\examples{
searchReferences(defineTextQuery("immun\%", partial=TRUE))[,c("name", "title")]
searchReferences(defineTextQuery("10090", field="taxonomy_id"))[,c("name", "title")]
searchReferences(
   defineTextQuery("10090", field="taxonomy_id") &
   defineTextQuery("immun\%", partial=TRUE)
)[,c("name", "title")]

}
\seealso{
\code{\link{surveyReferences}}, to easily obtain a listing of all available datasets.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surveyReferences.R
\name{surveyReferences}
\alias{surveyReferences}
\title{Survey reference metadata}
\usage{
surveyReferences(cache = cacheDirectory(), overwrite = FALSE, latest = TRUE)
}
\arguments{
\item{cache, overwrite}{Arguments to pass to \code{\link{fetchMetadataDatabase}}.}

\item{latest}{Whether to only consider the latest version of each dataset.}
}
\value{
A \linkS4class{DataFrame} where each row corresponds to a dataset, containing various columns of metadata.
Some columns may be lists to capture 1:many mappings.
}
\description{
Metadata survey for all available reference datasets in the \pkg{celldex} package.
}
\details{
The returned DataFrame contains the usual suspects like the title and description for each dataset,
the number of samples and types of labels, the organisms and genome builds involved, and so on.
More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}.
}
\examples{
surveyReferences()

}
\seealso{
\code{\link{searchReferences}}, to search on the metadata for specific datasets.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HumanPrimaryCellAtlasData.R
\name{HumanPrimaryCellAtlasData}
\alias{HumanPrimaryCellAtlasData}
\title{Obtain the HPCA data}
\usage{
HumanPrimaryCellAtlasData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of the data stored in
the Human Primary Cell Atlas. The data will be downloaded from ExperimentHub,
returning a \linkS4class{SummarizedExperiment} object for further use.
}
\details{
This function provides normalized expression values for 713 microarray samples from
the Human Primary Cell Atlas (HPCA) (Mabbott et al., 2013).
These 713 samples were processed and normalized as described in Aran, Looney and 
Liu et al. (2019).

Each sample has been assigned to one of 37 main cell types (\code{"label.main"}) 
and 157 subtypes (\code{"label.fine"}).
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"}, 
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- HumanPrimaryCellAtlasData()

}
\references{
Mabbott NA et al. (2013).
An expression atlas of human primary cells: inference of gene function from coexpression networks.
\emph{BMC Genomics} 14, Article 632. 

Aran D, Looney AP, Liu L et al. (2019). 
Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
\emph{Nat. Immunol.} 20, 163–172.
}
\author{
Friederike Dündar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatabaseImmuneExpressionData.R
\name{DatabaseImmuneCellExpressionData}
\alias{DatabaseImmuneCellExpressionData}
\title{Obtain human bulk RNA-seq data from DICE}
\usage{
DatabaseImmuneCellExpressionData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 1561 bulk RNA-seq samples
of sorted cell populations from the Database of Immune Cell Expression (DICE).
}
\details{
This function provides normalized expression values of 1561 bulk RNA-seq samples
generated by DICE from pure populations of human immune cells.

TPM normalized values for each cell type were downloaded from \url{https://dice-database.org/downloads}. 
Genes with no reads across samples were removed, and values were log2 normalized after a pseudocount of 1 was added.

The dataset contains 1561 human RNA-seq samples annotated to 5 main cell types (\code{"label.main"}):
\itemize{
    \item B cells
    \item Monocytes
    \item NK cells
    \item T cells, CD8+
    \item T cells, CD4+
}

Samples were additionally annotated to 15 fine cell types (\code{"label.fine"}):
\itemize{
    \item B cells, naive
    \item Monocytes, CD14+
    \item Monocytes, CD16+
    \item NK cells
    \item T cells, memory TREG
    \item T cells, CD4+, naive
    \item T cells, CD4+, naive, stimulated
    \item T cells, CD4+, naive Treg
    \item T cells, CD4+, Th1
    \item T cells, CD4+, Th1_17
    \item T cells, CD4+, Th2
    \item T cells, CD8+, naïve
    \item T cells, CD8+, naïve, stimulated
    \item T cells, CD4+, TFH
    \item T cells, CD4+, Th17
}
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- DatabaseImmuneCellExpressionData()

}
\references{
Schmiedel B et al. (2018).
Impact of Genetic Polymorphisms on Human Immune Cell Gene Expression.
\emph{Cell} 175, 1701-1715.
}
\author{
Jared Andrews
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NovershternHematopoieticData.R
\name{NovershternHematopoieticData}
\alias{NovershternHematopoieticData}
\title{Obtain bulk microarray expression for sorted hematopoietic cells}
\usage{
NovershternHematopoieticData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 211 bulk human microarray samples
of sorted hematopoietic cell populations that can be found in 
\href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE24759}{GSE24759}.
}
\details{
The dataset contains 211 human microarray samples annotated to 16 main cell types (\code{"label.main"}):
\itemize{
    \item Basophils
    \item B cells
    \item CMPs
    \item Dendritic cells
    \item Eosinophils
    \item Erythroid cells
    \item GMPS
    \item Granulocytes
    \item HSCs
    \item Megakaryocytes
    \item MEPs
    \item Monocytes
    \item NK cells
    \item NK T cells
    \item CD8+ T cells
    \item CD4+ T cells
}

Samples were additionally annotated to 38 fine cell types (\code{"label.fine"}):
\itemize{
    \item Basophils
    \item Naive B cells
    \item Mature B cells class able to switch
    \item Mature B cells
    \item Mature B cells class switched
    \item Common myeloid progenitors
    \item Plasmacytoid Dendritic Cells
    \item Myeloid Dendritic Cells
    \item Eosinophils
    \item Erythroid_CD34+ CD71+ GlyA-
    \item Erythroid_CD34- CD71+ GlyA-
    \item Erythroid_CD34- CD71+ GlyA+
    \item Erythroid_CD34- CD71lo GlyA+
    \item Erythroid_CD34- CD71- GlyA+
    \item Granulocyte/monocyte progenitors
    \item Colony Forming Unit-Granulocytes
    \item Granulocyte (Neutrophilic Metamyelocytes)
    \item Granulocyte (Neutrophils)
    \item Hematopoietic stem cells_CD133+ CD34dim
    \item Hematopoietic stem cell_CD38- CD34+
    \item Colony Forming Unit-Megakaryocytic
    \item Megakaryocytes
    \item Megakaryocyte/erythroid progenitors
    \item Colony Forming Unit-Monocytes
    \item Monocytes
    \item Mature NK cells_CD56- CD16+ CD3-
    \item Mature NK cells_CD56+ CD16+ CD3-
    \item Mature NK cells_CD56- CD16- CD3-
    \item NK T cells
    \item Early B cells
    \item Pro B cells
    \item CD8+ Effector Memory RA
    \item Naive CD8+ T cells
    \item CD8+ Effector Memory
    \item CD8+ Central Memory
    \item Naive CD4+ T cells
    \item CD4+ Effector Memory
    \item CD4+ Central Memory
}
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- NovershternHematopoieticData()

}
\references{
Novershtern N et al. (2011).
Densely interconnected transcriptional circuits control cell states in human hematopoiesis.
\emph{Cell} 144, 296-309.
}
\author{
Jared Andrews
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MonacoImmuneData.R
\name{MonacoImmuneData}
\alias{MonacoImmuneData}
\title{Obtain bulk RNA-seq data of sorted human immune cells}
\usage{
MonacoImmuneData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 114 bulk RNA-seq samples
of sorted immune cell populations that can be found in 
\href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE107011}{GSE107011}.
}
\details{
The dataset contains 114 human RNA-seq samples annotated to 10 main cell types (\code{"label.main"}):
\itemize{
    \item CD8+ T cells
    \item T cells
    \item CD4+ T cells
    \item Progenitors
    \item B cells
    \item Monocytes
    \item NK cells
    \item Dendritic cells
    \item Neutrophils
    \item Basophils
}

Samples were additionally annotated to 29 fine cell types (\code{"label.fine"}):
\itemize{
    \item Naive CD8 T cells
    \item Central memory CD8 T cells
    \item Effector memory CD8 T cells
    \item Terminal effector CD8 T cells
    \item MAIT cells
    \item Vd2 gd T cells
    \item Non-Vd2 gd T cells
    \item Follicular helper T cells
    \item T regulatory cells
    \item Th1 cells
    \item Th1/Th17 cells
    \item Th17 cells
    \item Th2 cells
    \item Naive CD4 T cells
    \item Terminal effector CD4 T cells
    \item Progenitor cells
    \item Naive B cells
    \item Non-switched memory B cells
    \item Exhausted B cells
    \item Switched memory B cells
    \item Plasmablasts
    \item Classical monocytes
    \item Intermediate monocytes
    \item Non classical monocytes
    \item Natural killer cells
    \item Plasmacytoid dendritic cells
    \item Myeloid dendritic cells
    \item Low-density neutrophils
    \item Low-density basophils
}
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- MonacoImmuneData()

}
\references{
Monaco G et al. (2019).
RNA-Seq Signatures Normalized by mRNA Abundance Allow Absolute Deconvolution of Human Immune Cell Types
\emph{Cell Rep.} 26, 1627-1640.
}
\author{
Jared Andrews
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveReference.R
\name{saveReference}
\alias{saveReference}
\title{Save a reference dataset}
\usage{
saveReference(x, labels, path, metadata)
}
\arguments{
\item{x}{Matrix of log-normalized expression values.
This may be sparse or dense, but should be non-integer and have no missing values.
Row names should be present and unique for all rows.}

\item{labels}{\linkS4class{DataFrame} of labels.
Each row of \code{labels} corresponds to a column of \code{x} and contains the label(s) for that column.
Each column of \code{labels} represents a different label type;
typically, the column name has a \code{label.} prefix to distinguish between, e.g., \code{label.fine}, \code{label.broad} and so on.
At least one column should be present.}

\item{path}{String containing the path to a directory in which to save \code{x}.}

\item{metadata}{Named list containing metadata for this reference dataset,
see the schema returned by \code{\link{fetchMetadataSchema}()}.
Note that the \code{applications.takane} property will be automatically added by this function and does not have to be supplied.}
}
\value{
\code{x} and \code{labels} are used to create a \linkS4class{SummarizedExperiment} that is saved into \code{path}.
\code{NULL} is invisibly returned.
}
\description{
Save a reference dataset to disk, usually in preparation for upload via \code{\link{uploadDirectory}}.
}
\details{
The SummarizedExperiment saved to \code{path} is guaranteed to have the \code{"logcounts"} assay and at least one column in \code{labels}.
This mirrors the expectation for reference datasets obtained via \code{\link{fetchReference}}.
}
\examples{
# Mocking up some data to be saved.
x <- matrix(rnorm(1000), ncol=10)
rownames(x) <- sprintf("GENE_\%i", seq_len(nrow(x)))
labels <- DataFrame(
    labels.broad = sample(c("B", "T", "NK"), ncol(x), replace=TRUE),
    labels.fine = sample(c("PC", "pre-B", "pro-B", "Th2", "CD4+T", "NK"), 
        ncol(x), replace=TRUE)
)

# Making up some metadata as well.
meta <- list(
    title="New reference dataset",
    description="This is a new reference dataset, generated from blah blah.",
    genome="GRCm38",
    taxonomy_id="10090",
    sources=list(
        list(provider="GEO", id="GSE123456"),
        list(provider="PubMed", id="123456"),
        list(provider="URL", id="https://reference.data.com", version="2024-02-26")
    ),
    maintainer_name="Chihaya Kisaragi",
    maintainer_email="kisaragi.chihaya@765.com"
)

# Actually saving it.
tmp <- tempfile()
saveReference(x, labels, tmp, meta)

# Reloading it to make sure it looks good.
alabaster.base::readObject(tmp)
str(jsonlite::fromJSON(file.path(tmp, "_bioconductor.json")))

}
\seealso{
\code{\link{uploadDirectory}}, to upload the saved dataset to the gypsum backend.

\code{\link{fetchReference}}, to download an existing dataset into the current sesssion.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listReferences.R
\name{listReferences}
\alias{listReferences}
\alias{listVersions}
\alias{fetchLatestVersion}
\title{List available references}
\usage{
listReferences()

listVersions(name)

fetchLatestVersion(name)
}
\arguments{
\item{name}{String containing the name of the reference dataset.}
}
\value{
For \code{listReferences}, a character vector containing the names of the available references.

For \code{listVersions}, a character vector containing the names of the available versions of the \code{name} reference.

For \code{fetchLatestVersion}, a string containing the name of the latest version.
}
\description{
List the available reference datasets and the associated versions in \pkg{celldex}.
}
\examples{
listReferences()
listVersions("immgen")
fetchLatestVersion("immgen")

}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetchReference.R
\name{fetchReference}
\alias{fetchReference}
\alias{fetchMetadata}
\title{Fetch a reference dataset}
\usage{
fetchReference(
  name,
  version,
  path = NA,
  package = "celldex",
  cache = cacheDirectory(),
  overwrite = FALSE,
  realize.assays = FALSE,
  ...
)

fetchMetadata(
  name,
  version,
  path = NA,
  package = "celldex",
  cache = cacheDirectory(),
  overwrite = FALSE
)
}
\arguments{
\item{name}{String containing the name of the reference dataset.}

\item{version}{String containing the version of the dataset.}

\item{path}{String containing the path to a subdataset, if \code{name} contains multiple reference datasets.
Defaults to \code{NA} if no subdatasets are present.}

\item{package}{String containing the name of the package.}

\item{cache, overwrite}{Arguments to pass to \code{\link[gypsum]{saveVersion}} or \code{\link[gypsum]{saveFile}}.}

\item{realize.assays}{Logical scalar indicating whether to realize assays into memory.
Dense and sparse \linkS4class{ReloadedArray} objects are converted into ordinary arrays and \linkS4class{dgCMatrix} objects, respectively.}

\item{...}{Further arguments to pass to \code{\link{readObject}}.}
}
\value{
\code{fetchReference} returns the dataset as a \linkS4class{SummarizedExperiment}.
This is guaranteed to have a \code{"logcounts"} assay with log-normalized expression values,
along with at least one character vector of labels in the column data.

\code{fetchMetadata} returns a named list of metadata for the specified dataset.
}
\description{
Fetch a reference dataset (or its metadata) from the gypsum backend.
}
\examples{
fetchReference("immgen", "2024-02-26")
str(fetchMetadata("immgen", "2024-02-26"))

}
\seealso{
\url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.

\code{\link{saveReference}} and \code{\link{uploadDirectory}}, to save and upload a dataset.

\code{\link{listReferences}} and \code{\link{listVersions}}, to get possible values for \code{name} and \code{version}.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MouseRNAseqData.R
\name{MouseRNAseqData}
\alias{MouseRNAseqData}
\title{Obtain mouse bulk expression data of sorted cell populations (RNA-seq)}
\usage{
MouseRNAseqData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 358 bulk RNA-seq samples
of sorted cell populations that can be found at GEO.
}
\details{
This dataset was contributed by the Benayoun Lab that identified, 
downloaded and processed data sets on GEO that corresponded to sorted cell
types (Benayoun et al., 2019).

The dataset contains 358 mouse RNA-seq samples annotated to 18 main cell types (\code{"label.main"}):
\itemize{
    \item Adipocytes
    \item Astrocytes
    \item B cells 
    \item Cardiomyocytes
    \item Dendritic cells
    \item Endothelial cells
    \item Epithelial cells
    \item Erythrocytes
    \item Fibroblasts
    \item Granulocytes
    \item Hepatocytes
    \item Macrophages
    \item Microglia
    \item Monocytes
    \item Neurons
    \item NK cells
    \item Oligodendrocytes
    \item T cells
} 
These are split further into 28 subtypes (\code{"label.fine"}).
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- MouseRNAseqData()

}
\references{
Benayoun B et al. (2019).
Remodeling of epigenome and transcriptome landscapes with aging in mice reveals widespread induction of inflammatory responses.
\emph{Genome Res.} 29, 697-709.

Code at \url{https://github.com/BenayounLaboratory/Mouse_Aging_Epigenomics_2018/tree/master/FigureS7_CIBERSORT/RNAseq_datasets_for_Deconvolution/2017-01-18}
}
\author{
Friederike Dündar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BlueprintEncodeData.R
\name{BlueprintEncodeData}
\alias{BlueprintEncodeData}
\title{Obtain human bulk RNA-seq data from Blueprint and ENCODE}
\usage{
BlueprintEncodeData(
  rm.NA = c("rows", "cols", "both", "none"),
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{rm.NA}{String specifying how missing values should be handled.
\code{"rows"} will remove genes with at least one missing value, 
\code{"cols"} will remove samples with at least one missing value,
\code{"both"} will remove any gene or sample with at least one missing value,
and \code{"none"} will not perform any removal.}

\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 259 RNA-seq samples of
pure stroma and immune cells as generated and supplied by Blueprint and ENCODE.
}
\details{
This function provides normalized expression values for 259 bulk RNA-seq samples
generated by Blueprint and ENCODE from pure populations of stroma and immune 
cells (Martens and Stunnenberg, 2013; The ENCODE Consortium, 2012).
The samples were processed and normalized as described in Aran, Looney and
Liu et al. (2019), i.e., the raw RNA-seq counts were downloaded from 
Blueprint and ENCODE in 2016 and normalized via edgeR (TPMs).

Blueprint Epigenomics contains 144 RNA-seq pure immune samples annotated to 28 cell types.
ENCODE contains 115 RNA-seq pure stroma and immune samples annotated to 17 cell types.
All together, this reference contains 259 samples with 43 cell types (\code{"label.fine"}),
manually aggregated into 24 broad classes (\code{"label.main"}).
The fine labels have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- BlueprintEncodeData(rm.NA = "rows")

}
\references{
The ENCODE Project Consortium (2012).
An integrated encyclopedia of DNA elements in the human genome.
\emph{Nature} 489, pages 57–74.

Martens JHA and Stunnenberg HG (2013). 
BLUEPRINT: mapping human blood cell epigenomes.
\emph{Haematologica} 98, 1487–1489.

Aran D, Looney AP, Liu L et al. (2019). 
Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
\emph{Nat. Immunol.} 20, 163–172.
}
\author{
Friederike Dündar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ImmGenData.R
\name{ImmGenData}
\alias{ImmGenData}
\title{Obtain mouse bulk expression data from the Immunologic Genome Project}
\usage{
ImmGenData(
  ensembl = FALSE,
  cell.ont = c("all", "nonna", "none"),
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether to convert row names to Ensembl IDs.
Genes without a mapping to a non-duplicated Ensembl ID are discarded.}

\item{cell.ont}{String specifying whether Cell Ontology terms should be included in the \code{\link{colData}}.
If \code{"nonna"}, all samples without a valid term are discarded;
if \code{"all"}, all samples are returned with (possibly \code{NA}) terms;
if \code{"none"}, terms are not added.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SummarizedExperiment} object with a \code{"logcounts"} assay
containing the log-normalized expression values, along with cell type labels in the 
\code{\link{colData}}.
}
\description{
Download and cache the normalized expression values of 830 microarray samples of
pure mouse immune cells, generated by the Immunologic Genome Project (ImmGen).
}
\details{
This function provides normalized expression values of 830 microarray samples
generated by ImmGen from pure populations of murine immune cells (<http://www.immgen.org/>).
The samples were processed and normalized as described in Aran, Looney and
Liu et al. (2019), i.e., CEL files from the Gene Expression Omnibus (GEO; GSE15907 and GSE37448), 
were downloaded, processed, and normalized using the robust multi-array average
(RMA) procedure on probe-level data.

This dataset consists of 20 broad cell types (\code{"label.main"}) and 253
finely resolved cell subtypes (\code{"label.fine"}).
The subtypes have also been mapped to the Cell Ontology (\code{"label.ont"},
if \code{cell.ont} is not \code{"none"}), which can be used for further programmatic
queries.
}
\examples{
ref.se <- ImmGenData()

}
\references{
Heng TS, Painter MW, Immunological Genome Project Consortium (2008).
The Immunological Genome Project: networks of gene expression in immune cells.
\emph{Nat. Immunol.} 9, 1091-1094. 

Aran D, Looney AP, Liu L et al. (2019). 
Reference-based analysis of lung single-cell sequencing reveals a transitional profibrotic macrophage.
\emph{Nat. Immunol.} 20, 163–172.
}
\author{
Friederike Dündar
}
